﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
namespace ScrabbleClient
{
    /// <summary>
    /// Lógica de interacción para App.xaml
    /// </summary>
    public partial class App : Application
    {
        /* Aquestes tres funcions tenen la missió de fer que, quan un textBox rebi el focus, s'autoseleccioni tot el text com sol ser
         * habitual en el funcionament de les aplicacions windows.
         * Amb WPF, es veu que no es fa per defecte i la seva implementació mes senzilla que he trobat es mitjançant aquestes funcions
         * http://www.intertech.com/Blog/Post/How-to-Select-All-Text-in-a-WPF-Content-on-Focus.aspx
         */
        protected override void OnStartup(StartupEventArgs e) 
        {
            EventManager.RegisterClassHandler(typeof(TextBox), TextBox.PreviewMouseLeftButtonDownEvent, new MouseButtonEventHandler(SelectivelyHandleMouseButton), true);
            EventManager.RegisterClassHandler(typeof(TextBox), TextBox.GotKeyboardFocusEvent, new RoutedEventHandler(SelectAllText), true); 
            
            base.OnStartup(e); 
        } 
        
        private static void SelectivelyHandleMouseButton(object sender, MouseButtonEventArgs e) 
        { 
            var textbox = (sender as TextBox);            
            if (textbox != null && !textbox.IsKeyboardFocusWithin) 
            {                
                if (e.OriginalSource.GetType().Name == "TextBoxView") 
                { 
                    e.Handled = true; textbox.Focus(); 
                } 
            } 
        } 
        
        private static void SelectAllText(object sender, RoutedEventArgs e) 
        { 
            var textBox = e.OriginalSource as TextBox; 
            if (textBox != null) textBox.SelectAll(); 
        }

    }
}
