﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using ScrabbleClient.Domain;
using ServerLibrary.WCF.DTO;

namespace ScrabbleClient.Interface
{
    /// <summary>
    /// Lógica de interacción para CurrentGames.xaml
    /// </summary>
    public partial class CurrentGames : Window, IGameObserver
    {
        private List<GameInfoDTO> games;
        public CurrentGames()
        {
            InitializeComponent();
        }

        private void frmNewGame_Loaded(object sender, RoutedEventArgs e)
        {
            try
            {
                GetCurrentGames();

                //Registrem la finestra com a observadora dels events que el server informi i que distribuïrà el clientManager
                RegisterAsGameObserver();
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void frmCurrentGames_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            try
            {
                UnregisterAsGameObserver();
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void RegisterAsGameObserver()
        {
            ClientManager.Instance.AddObserver(this);
        }

        private void UnregisterAsGameObserver()
        {
            ClientManager.Instance.RemoveObserver(this);
        }

        private void GetCurrentGames()
        {
            games = ClientManager.Instance.GetCurrentGames();
            dgGames.ItemsSource = games;
        }

        public void YourTurn(GameInfoDTO gameInfo, List<List<BoxDTO>> boardInfo, List<TileDTO> rackInfo, MoveInfoDTO moveInfo)
        {
            //Busquem la partida a la llista i actualitzem
            int index = games.FindIndex(g => g.Id == gameInfo.Id);

            if (index > -1)
            {
                games[index] = gameInfo;
            }

            dgGames.Items.Refresh();
        }

        public void OpponentResigned(GameInfoDTO gameInfo)
        {
            //Busquem la partida a la llista i actualitzem
            int index = games.FindIndex(g => g.Id == gameInfo.Id);

            if (index > -1)
            {
                games[index] = gameInfo;
            }

            dgGames.Items.Refresh();
        }

        private void btnBack_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        private void btnPlayGame_Click(object sender, RoutedEventArgs e)
        {
            if (dgGames.SelectedItem == null)
            {
                MessageBox.Show(this, "No hi ha cap partida seleccionada", "Error");
                return;
            }

            try
            {
                GameInfoDTO gameInfo = (GameInfoDTO)dgGames.SelectedItem;

                //No es pot jugar una partida que no tingui jugador 2
                if (gameInfo.Player2 == null)
                {
                    MessageBox.Show(this, "No es pot jugar la partida perquè falta un segon jugador", "Error");
                    return;
                }

                PlayGame pg = new PlayGame();         
                pg.Owner = this;
                pg.SetGame(gameInfo, false);
                pg.ShowDialog();
                GetCurrentGames();
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void btnDeleteGame_Click(object sender, RoutedEventArgs e)
        {
            if (dgGames.SelectedItem == null)
            {
                MessageBox.Show(this, "No hi ha cap partida seleccionada", "Error");
                return;
            }

            if (MessageBox.Show(this, "Estas segur que vols eliminar la partida?", "Confirmació", MessageBoxButton.YesNo, MessageBoxImage.Question) != MessageBoxResult.Yes)
            {
                return;
            }

            try
            {
                ClientManager.Instance.DeleteGame(((GameInfoDTO)dgGames.SelectedItem).Id);
                GetCurrentGames();
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void btnRefresh_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                GetCurrentGames();
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }            
        }       
    }
}
