﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using ServerLibrary.WCF.DTO;
using ServerLibrary.Domain.Boards;
using ScrabbleClient.Domain;

namespace ScrabbleClient.Interface
{
    /// <summary>
    /// Lógica de interacción para EditBoard.xaml
    /// </summary>
    public partial class EditBoard : Window
    {
        private string id;
        private bool changed = false;
        private List<List<BoxDTO>> boxes;

        public EditBoard()
        {
            InitializeComponent();          
        }

        public void NewBoard()
        {
            //Setejem els valors per defecte per un tauler nou
            id = "";
            txbBoard.Text = "";

            //Creem la matriu de caselles
            boxes = new List<List<BoxDTO>>();
            
            for (int i = 0; i < 15; i++)
            {
                boxes.Add(new List<BoxDTO>());

                for (int j = 0; j < 15; j++)
                {
                    BoxDTO box = new BoxDTO() { Id= "", BoxType = (int)Box.BoxType.Normal };
                 
                    boxes[i].Add(box);

                    //Accedim a les caselles del grid per fila i columna mitjançant una consulta LINQ i l'associem a l'objecte intern
                    Image imgBox = grdBoard.Children.Cast<Image>().First(e => Grid.GetRow(e) == i && Grid.GetColumn(e) == j);
                    imgBox.Tag = box;
                }
            }

            changed = false;
        }

        public void LoadBoard(BoardDTO board) 
        {
            id = board.Id;
            txbBoard.Text = board.Name;

            boxes = ClientManager.Instance.GetBoardBoxes(id);

            for (int i = 0; i < 15; i++)
            {
                for (int j = 0; j < 15; j++)
                {
                    //Accedim a les caselles del grid per fila i columna mitjançant una consulta LINQ i l'associem a l'objecte intern
                    //I l'imatge que correspongui
                    Image box = grdBoard.Children.Cast<Image>().First(e => Grid.GetRow(e) == i && Grid.GetColumn(e) == j);
                    box.Source = GetImageSource((Box.BoxType)boxes[i][j].BoxType);
                    box.Tag = boxes[i][j];
                }
            }

            changed = false;
        }

        private ImageSource GetImageSource(Box.BoxType boxType)
        {
            switch (boxType)
            {
                case Box.BoxType.Normal: 
                    return imgN.Source;

                case Box.BoxType.Initial:
                    return imgI.Source;

                case Box.BoxType.DoubleLetter:
                    return imgDL.Source;

                case Box.BoxType.TripleLetter:
                    return imgTL.Source;

                case Box.BoxType.DoubleWord:
                    return imgDP.Source;

                case Box.BoxType.TripleWord:
                    return imgTP.Source;

                default:
                    return null;
            }
        }


        private void SaveBoard()
        {
            if (txbBoard.Text == "")
            {
                MessageBox.Show(this, "El nom del tauler no pot estar en blanc", "Error");
                txbBoard.Focus();
                return;
            }
            if (txbBoard.Text.Length > 100)
            {
                MessageBox.Show(this, "El nom del tauler no pot superar els 100 caracters", "Error");
                txbBoard.Focus();
                return;
            }

            BoardDTO board = new BoardDTO();
            board.Id = id;
            board.Name = txbBoard.Text;

            ClientManager.Instance.SaveBoard(board, boxes);
            Close();
        }

        

        private void txbBoard_TextChanged(object sender, TextChangedEventArgs e)
        {
            changed = true;
        }

        private void btnSave_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                SaveBoard();            
            }
            catch(Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void btnCancel_Click(object sender, RoutedEventArgs e)
        {
            //Si hem editat o canviat quelcom, demanem confirmació
            if (changed)
            {
                if (MessageBox.Show(this, "Estas segur que vols sortir sense desar?", "Confirmació", MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes)
                {
                    Close();
                }
            }
            else
            {
                Close();
            }
        }

        /*Gestió de Drag & drop*/
        private void img_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            Image image = e.Source as Image;
            DataObject data2 = new DataObject(typeof(ImageSource), image.Source);
            DragDropData data = new DragDropData() { ImgSrc = image.Source, Tag = image.Tag };
            DragDrop.DoDragDrop(image, data, DragDropEffects.Copy);
        }

        private void img_Drop(object sender, DragEventArgs e)
        {
            DragDropData data = e.Data.GetData(typeof(DragDropData)) as DragDropData;
            ImageSource image = data.ImgSrc;
            Image imageDest = sender as Image;
            imageDest.Source = image;
            ((BoxDTO)imageDest.Tag).BoxType = Convert.ToInt32((string)data.Tag);
            changed = true;
        }

        private class DragDropData
        {
            public ImageSource ImgSrc { get; set; }

            public Object Tag { get; set; }
        }

        private void btnHelp_Click(object sender, RoutedEventArgs e)
        {
            MessageBox.Show(this, "Arrossega les caselles de la dreta a la casella que desitgis dins del tauler.\nHi pot haver tantes caselles especials com vulguis, pero una i nomes una casella inicial.", "Informació", MessageBoxButton.OK, MessageBoxImage.Information);
        }
    }
}
