﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using ScrabbleClient.Domain;
using ServerLibrary.WCF.DTO;

namespace ScrabbleClient.Interface
{
    /// <summary>
    /// Lógica de interacción para EditDictionaryInfo.xaml
    /// </summary>
    public partial class EditDictionaryInfo : Window
    {
        private string id;
        private bool changed = false;

        public EditDictionaryInfo()
        {
            InitializeComponent();
            txbName.Focus();
        }

        public void NewDictionaryInfo()
        {
            //Setejem els valors per defecte per un diccionari nou
            id = "";
            txbName.Text = "";
            txbFileName.Text = "";

            changed = false;
        }

        public void LoadDictionaryInfo(DictionaryInfoDTO dictionary)
        {
            //Carreguem les dades d'un diccionari existent
            id = dictionary.Id;
            txbName.Text = dictionary.Name;
            txbFileName.Text = dictionary.FileName;

            changed = false;
        }

        private void SaveDictionaryInfo()
        {
            if (txbName.Text == "")
            {
                MessageBox.Show(this, "El nom del diccionari no pot estar en blanc", "Error");
                txbName.Focus();
                return;
            }

            if (txbName.Text.Length > 100)
            {
                MessageBox.Show(this, "El nom del diccionari no pot superar els 100 caracters", "Error");
                txbName.Focus();
                return;
            }

            if (txbFileName.Text == "")
            {
                MessageBox.Show(this, "L'arxiu associat no pot estar en blanc", "Error");
                txbFileName.Focus();
                return;
            }

            if (txbFileName.Text.Length > 100)
            {
                MessageBox.Show(this, "L'arxiu associat no pot superar els 100 caracters", "Error");
                txbFileName.Focus();
                return;
            }

            DictionaryInfoDTO dictionary = new DictionaryInfoDTO();
            dictionary.Id = id;
            dictionary.Name = txbName.Text;
            dictionary.FileName = txbFileName.Text;

            ClientManager.Instance.SaveDictionary(dictionary);

            Close();
        }

        private void btnCancel_Click(object sender, RoutedEventArgs e)
        {
            //Si hem editat o canviat quelcom, demanem confirmació
            if (changed)
            {
                if (MessageBox.Show(this, "Estas segur que vols sortir sense desar?", "Confirmació", MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes)
                {
                    Close();
                }
            }
            else
            {
                Close();
            }
        }

        private void btnSave_Click(object sender, RoutedEventArgs e)
        {        
            try
            {
                SaveDictionaryInfo();
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }


        private void txbName_TextChanged(object sender, TextChangedEventArgs e)
        {
            changed = true;
        }
    }
}
