﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using ServerLibrary.WCF.DTO;
using ServerLibrary.Domain.Boards;
using ServerLibrary.Domain.TileSets;
using ScrabbleClient.Domain;

namespace ScrabbleClient.Interface
{
    /// <summary>
    /// Lógica de interacción para EditTileSet.xaml
    /// </summary>
    public partial class EditTileSet : Window
    {
        private string id;
        private bool changed = false;
        private List<TileDTO> tiles;

        public EditTileSet()
        {
            InitializeComponent();          
        }

        public void NewTileSet()
        {
            //Setejem els valors per defecte per un sac nou
            id = "";
            txbTileSet.Text = "";

            //Creem la matriu de caselles
            tiles = new List<TileDTO>();
            
            for (int i = 0; i < 100; i++)
            {
                TileDTO tile = new TileDTO() { Id = "", Letter = "", Value = 1, Type = (int)Tile.TileType.Normal };
                tiles.Add(tile);

                //Accedim a les caselles del grid per fila i columna mitjançant una consulta LINQ i l'associem a l'objecte intern
                TextBox txbTile = grdTileSet.Children.Cast<TextBox>().First(e => Grid.GetRow(e) == (i/10) && Grid.GetColumn(e) == (i%10));
                txbTile.Tag = tile;
            }

            changed = false;
        }

        public void LoadTileSet(TileSetDTO tileSet) 
        {
            id = tileSet.Id;
            txbTileSet.Text = tileSet.Name;

            tiles = ClientManager.Instance.GetTileSetTiles(id);

            for (int i = 0; i < 100; i++)
            {
                TileDTO tile = tiles[i];

                //Accedim a les caselles del grid per fila i columna mitjançant una consulta LINQ i l'associem a l'objecte intern
                TextBox txbTile = grdTileSet.Children.Cast<TextBox>().First(e => Grid.GetRow(e) == (i / 10) && Grid.GetColumn(e) == (i % 10));
                txbTile.Tag = tile;
                txbTile.Background = new ImageBrush(GetImageSource(tile.Value));
                txbTile.Text = tile.Letter;
            }

            changed = false;
        }

        private ImageSource GetImageSource(int value)
        {
            switch (value)
            {
                case 0:
                    return img0.Source;

                case 1: 
                    return img1.Source;

                case 2:
                    return img2.Source;

                case 3:
                    return img3.Source;

                case 4:
                    return img4.Source;

                case 5:
                    return img5.Source;

                case 8:
                    return img8.Source;

                case 10:
                    return img10.Source;

                default:
                    return null;
            }
        }


        private void SaveTileSet()
        {
            if (txbTileSet.Text == "")
            {
                MessageBox.Show(this, "El nom del sac de fitxes no pot estar en blanc", "Error");
                txbTileSet.Focus();
                return;
            }
            if (txbTileSet.Text.Length > 100)
            {
                MessageBox.Show(this, "El nom del sac de fitxes no pot superar els 100 caracters", "Error");
                txbTileSet.Focus();
                return;
            }

            //Passem les lletres del grid als objectes corresponents
            for (int i = 0; i < 10; i++)
            {
                for (int j = 0; j < 10; j++)
                {
                    TextBox txbTile = grdTileSet.Children.Cast<TextBox>().First(e => Grid.GetRow(e) == i && Grid.GetColumn(e) == j);                

                    TileDTO tile = (TileDTO)txbTile.Tag;
                    tile.Letter = txbTile.Text;
                    if (tile.Letter == "")
                    {
                        tile.Type = (int)Tile.TileType.Blank;
                        tile.Value = 0;
                    }
                }
            }

            TileSetDTO tileSet = new TileSetDTO();
            tileSet.Id = id;
            tileSet.Name = txbTileSet.Text;

            ClientManager.Instance.SaveTileSet(tileSet, tiles);
            Close();
        }
       

        private void txbBoard_TextChanged(object sender, TextChangedEventArgs e)
        {
            changed = true;
        }

        private void btnSave_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                SaveTileSet();            
            }
            catch(Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void btnCancel_Click(object sender, RoutedEventArgs e)
        {
            //Si hem editat o canviat quelcom, demanem confirmació
            if (changed)
            {
                if (MessageBox.Show(this, "Estas segur que vols sortir sense desar?", "Confirmació", MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes)
                {
                    Close();
                }
            }
            else
            {
                Close();
            }
        }

        private void btnHelp_Click(object sender, RoutedEventArgs e)
        {
            MessageBox.Show(this, "Arrossega les caselles de la dreta a les fitxes que desitgis dins del tauler per assignar-los-hi un valor.\n" +
                            "Escriu a cada fitxa la lletra desitjada, una fitxa sense lletra serà considerada un escarràs.\n" +
                            "Cada fitxa pot estar formada per lletres de fins a 3 caràcters (per exemple NY o L·L)" +
                            "És responsabilitat teva afegir lletres que formin paraules vàlides en el teu idioma.", "Informació", MessageBoxButton.OK, MessageBoxImage.Information);
        }

        /*Gestió de Drag & drop*/
        private void img_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            Image image = e.Source as Image;
            DataObject data2 = new DataObject(typeof(ImageSource), image.Source);
            DragDropData data = new DragDropData() { ImgSrc = image.Source, Tag = image.Tag };
            DragDrop.DoDragDrop(image, data, DragDropEffects.Copy);
        }

        private void txbTile_PreviewDragEnter(object sender, DragEventArgs e)
        {
            e.Effects = DragDropEffects.Copy;
            e.Handled = true;
        }

        private void txbTile_PreviewDrop(object sender, DragEventArgs e)
        {
            DragDropData data = e.Data.GetData(typeof(DragDropData)) as DragDropData;
            ImageSource image = data.ImgSrc;
            TextBox txbDest = sender as TextBox;
            ((ImageBrush)txbDest.Background).ImageSource = image;

            ((TileDTO)txbDest.Tag).Value = Convert.ToInt32((string)data.Tag);
            changed = true;
        }

        private class DragDropData
        {
            public ImageSource ImgSrc { get; set; }

            public Object Tag { get; set; }
        }
    }
}
