﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using ScrabbleClient.Domain;
using System.Web.Security;

namespace ScrabbleClient.Interface
{
    /// <summary>
    /// Lógica de interacción para EditUser.xaml
    /// </summary>
    public partial class EditUser : Window
    {
        private bool changed = false;

        public EditUser()
        {
            InitializeComponent();
            txbUser.Text = ClientManager.Instance.User.Name;
            txbUser.Focus();
            changed = false;
        }

        private void btnCancel_Click(object sender, RoutedEventArgs e)
        {
            //Si hem editat o canviat quelcom, demanem confirmació
            if (changed)
            {
                if (MessageBox.Show(this, "Estas segur que vols sortir sense desar?", "Confirmació", MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes)
                {
                    Close();
                }
            }
            else
            {
                Close();
            }
        }

        private void btnSave_Click(object sender, RoutedEventArgs e)
        {
            if (txbUser.Text == "")
            {
                MessageBox.Show(this, "El nom d'usuari no pot estar en blanc", "Error");
                txbUser.Focus();
                return;
            }

            if (pwbPassword.Password == "")
            {
                MessageBox.Show(this, "La contrasenya no pot estar en blanc", "Error");
                pwbPassword.Focus();
                return;
            }

            if (pwbRepeatPassword.Password != pwbPassword.Password)
            {
                MessageBox.Show(this, "Les contrasenyes no coincideixen", "Error");
                pwbPassword.Focus();
                return;
            }
            UpdateUser(txbUser.Text, pwbPassword.Password);
        }

        private void UpdateUser(string name, string password)
        {
            try
            {
                //La funció HashPasswordForStoringInConfigFile esta obsoleta i aixó genera un warning. 
                //Tan mateix per aquest projecte només es pretén implementar un mètode senzill de protecció de contrasenyes
                //i aquesta funció proporciona una implementació ràpida i fàcil.
                //Es per aixó que s'inclou aquesta directiva de compilació tan sols per aquestes linies, per evitar el warning.
                #pragma warning disable 0618
                string hashedPassword = FormsAuthentication.HashPasswordForStoringInConfigFile(password, "MD5");
                #pragma warning restore 0618

                ClientManager cm = ClientManager.Instance;

                cm.UpdateUser(name, hashedPassword);

                MessageBox.Show(this, "Usuari modificat correctament", "Informació");

                this.Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                txbUser.Focus();
            }
        }

        private void txbName_TextChanged(object sender, TextChangedEventArgs e)
        {
            changed = true;
        }

        private void txbPassword_PasswordChanged(object sender, RoutedEventArgs e)
        {
            changed = true;
        }
    }
}
