﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Web.Security;
using ScrabbleClient.Domain;

namespace ScrabbleClient.Interface
{
    /// <summary>
    /// Lógica de interacción para MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();          
        }


        /*
         * ************************************************************************
         * Gestió del canvas Inicial (Login, crear nou usuari...)
         * ************************************************************************
         */

        private void Login(string name, string password)
        {
            try
            {
                //La funció HashPasswordForStoringInConfigFile esta obsoleta i aixó genera un warning. 
                //Tan mateix per aquest projecte només es pretén implementar un mètode senzill de protecció de contrasenyes
                //i aquesta funció proporciona una implementació ràpida i fàcil.
                //Es per aixó que s'inclou aquesta directiva de compilació tan sols per aquestes linies, per evitar el warning.
                #pragma warning disable 0618
                string hashedPassword = FormsAuthentication.HashPasswordForStoringInConfigFile(password, "MD5");
                #pragma warning restore 0618

                ClientManager cm = ClientManager.Instance;

                cm.Login(name, hashedPassword);

                //MessageBox.Show("Sessió iniciada correctament.", "Informació");
                if (cm.User.IsAdmin)
                {
                    btnConfigureDictionaryInfo.Visibility = Visibility.Visible;
                }
                else
                {
                    btnConfigureDictionaryInfo.Visibility = Visibility.Hidden;
                }
                cnvMain.Visibility = System.Windows.Visibility.Visible;
                cnvInitial.Visibility = System.Windows.Visibility.Hidden;
                ClearControlsData();
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void CreateUser(string name, string password)
        {
            try
            {
                //La funció HashPasswordForStoringInConfigFile esta obsoleta i aixó genera un warning. 
                //Tan mateix per aquest projecte només es pretén implementar un mètode senzill de protecció de contrasenyes
                //i aquesta funció proporciona una implementació ràpida i fàcil.
                //Es per aixó que s'inclou aquesta directiva de compilació tan sols per aquestes linies, per evitar el warning.
                #pragma warning disable 0618
                string hashedPassword = FormsAuthentication.HashPasswordForStoringInConfigFile(password, "MD5");
                #pragma warning restore 0618

                ClientManager cm = ClientManager.Instance;

                cm.CreateUser(name, hashedPassword);

                MessageBox.Show(this, "Usuari creat correctament", "Informació");

                if (cm.User.IsAdmin)
                {
                    btnConfigureDictionaryInfo.Visibility = Visibility.Visible;
                }
                else
                {
                    btnConfigureDictionaryInfo.Visibility = Visibility.Hidden;
                }
                cnvMain.Visibility = System.Windows.Visibility.Visible;
                cnvInitial.Visibility = System.Windows.Visibility.Hidden;
                ClearControlsData();
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void btnNewUser_Click(object sender, RoutedEventArgs e)
        {
            if (txbNewUser.Text == "")
            {
                MessageBox.Show(this, "El nom d'usuari no pot estar en blanc", "Error");
                txbNewUser.Focus();
                return;
            }

            if (pwbNewPassword.Password == "")
            {
                MessageBox.Show(this, "La contrasenya no pot estar en blanc", "Error");
                pwbNewPassword.Focus();
                return;
            }

            if (pwbRepeatPassword.Password != pwbNewPassword.Password)
            {
                MessageBox.Show(this, "Les contrasenyes no coincideixen", "Error");
                pwbNewPassword.Focus();
                return;
            }
            CreateUser(txbNewUser.Text, pwbNewPassword.Password);
        }

        private void btnLogin_Click(object sender, RoutedEventArgs e)
        {
            if (txbRegisteredUser.Text == "")
            {
                MessageBox.Show(this, "El nom d'usuari no pot estar en blanc", "Error");
                txbRegisteredUser.Focus();
                return;
            }

            if (pwbRegisteredPassword.Password == "")
            {
                MessageBox.Show(this, "La contrasenya no pot estar en blanc", "Error");
                pwbRegisteredPassword.Focus();
                return;
            }
            Login(txbRegisteredUser.Text, pwbRegisteredPassword.Password);
        }

        private void btnExit_Click(object sender, RoutedEventArgs e)
        {
            Close();
            Application.Current.Shutdown();
        }

        private void txbRegisteredUser_GotFocus(object sender, RoutedEventArgs e)
        {
            SetButtonLoginDefault();
        }

        private void txbNewUser_GotFocus(object sender, RoutedEventArgs e)
        {
            SetButtonNewUserDefault();
        }

        private void SetButtonLoginDefault()
        {
            btnNewUser.IsDefault = false;
            btnLogin.IsDefault = true;
        }

        private void SetButtonNewUserDefault()
        {
            btnNewUser.IsDefault = true;
            btnLogin.IsDefault = false;
        }

        private void ClearControlsData()
        {
            txbRegisteredUser.Text = "";
            pwbRegisteredPassword.Password = "";
            txbNewUser.Text = "";
            pwbNewPassword.Password = "";
            pwbRepeatPassword.Password = "";
            txbRegisteredUser.Focus();
        }

        private void frmInitialWindow_Closed(object sender, EventArgs e)
        {
            Application.Current.Shutdown();
        }

        /*
         * ************************************************************************
         * Gestió del canvas Principal
         * ************************************************************************
         */

        private void btnCloseSession_Click(object sender, RoutedEventArgs e)
        {
            if (MessageBox.Show(this, "Estas segur que vols tancar la sessió?", "Scrabble", MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes)
            {
                ClientManager.Instance.Logout();
                cnvInitial.Visibility = System.Windows.Visibility.Visible;
                cnvMain.Visibility = System.Windows.Visibility.Hidden;
            }
        }

        private void btnEditUser_Click(object sender, RoutedEventArgs e)
        {
            EditUser ed = new EditUser();
            ed.Owner = this;
            ed.ShowDialog();
        }

        private void OpenWindow(Window window)
        {
            this.Visibility = System.Windows.Visibility.Hidden;
            window.Owner = this;
            window.ShowDialog();
            this.Visibility = System.Windows.Visibility.Visible;
        }

        private void btnConfigureBoard_Click(object sender, RoutedEventArgs e)
        {
            EditBoards window = new EditBoards();
            OpenWindow(window);
        }

        private void btnConfigureTileSet_Click(object sender, RoutedEventArgs e)
        {
            EditTileSets window = new EditTileSets();
            OpenWindow(window);
        }

        private void btnConfigureDictionary_Click(object sender, RoutedEventArgs e)
        {
            EditDictionaryInfos window = new EditDictionaryInfos();
            OpenWindow(window);
        }

        private void btnNewGame_Click(object sender, RoutedEventArgs e)
        {
            NewGame window = new NewGame();
            OpenWindow(window);
        }

        private void btnSearchGame_Click(object sender, RoutedEventArgs e)
        {
            FindGame window = new FindGame();
            OpenWindow(window);
        }

        private void btnCurrentGames_Click(object sender, RoutedEventArgs e)
        {
            CurrentGames window = new CurrentGames();
            OpenWindow(window);
        }

        private void btnShowStats_Click(object sender, RoutedEventArgs e)
        {
            ShowStats window = new ShowStats();
            OpenWindow(window);
        }

        private void btnDeleteUserStats_Click(object sender, RoutedEventArgs e)
        {
            if (MessageBox.Show(this, "Estas segur que vols eliminar les teves classificacions?", "Confirmació", MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes)
            {
                try
                {
                    ClientManager.Instance.DeleteUserStats();
                    MessageBox.Show(this, "Classificacions eliminades correctament", "Informació", MessageBoxButton.OK, MessageBoxImage.Information);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(this, ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }
        }

        private void frmMainWindow_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            //Ens assegurem que es fa un logout si no s'havia fet
            try
            {
                ClientManager.Instance.Logout();
            }
            catch(Exception)
            {
            }
        }

        private void btnDeleteUser_Click(object sender, RoutedEventArgs e)
        {         
            if (MessageBox.Show(this, "Estas segur que vols eliminar el teu usuari?", "Confirmació", MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes)
            {
                bool deleteStats = (MessageBox.Show(this, "Vols eliminar també les teves classificacions?", "Confirmació", MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes);

                try
                {
                    ClientManager.Instance.DeleteUser(deleteStats);
                    MessageBox.Show(this, "Usuari eliminat correctament.\nHa sigut un plaer jugar amb tu.\nFins aviat!", "Informació", MessageBoxButton.OK, MessageBoxImage.Information);
                    ClientManager.Instance.Logout();
                    cnvInitial.Visibility = System.Windows.Visibility.Visible;
                    cnvMain.Visibility = System.Windows.Visibility.Hidden;
                }
                catch (Exception ex)
                {
                    MessageBox.Show(this, ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                }
            }
        }
    }
}
