﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using ServerLibrary.WCF.DTO;
using ScrabbleClient.Domain;

namespace ScrabbleClient.Interface
{
    /// <summary>
    /// Lógica de interacción para NewGame.xaml
    /// </summary>
    public partial class NewGame : Window
    {
        private List<BoardDTO> boards;
        private List<TileSetDTO> tileSets; 
        private List<DictionaryInfoDTO> dictionaries;

        public NewGame()
        {
            InitializeComponent();
        }

        public void LoadData()
        {
            txbName.Text = DateTime.UtcNow.ToString(); 
            boards = ClientManager.Instance.GetBoards();
            tileSets = ClientManager.Instance.GetTileSets();            
            dictionaries = ClientManager.Instance.GetDictionaries();
            cbbBoard.ItemsSource = boards;
            cbbTileSet.ItemsSource = tileSets;
            cbbDictionary.ItemsSource = dictionaries;

            if (cbbBoard.Items.Count > 0)
            { 
                cbbBoard.SelectedIndex = 0; 
            }

            if (cbbTileSet.Items.Count > 0)
            {
                cbbTileSet.SelectedIndex = 0;
            }

            if (cbbDictionary.Items.Count > 0)
            {
                cbbDictionary.SelectedIndex = 0;
            }

        }

        private void btnBack_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            try
            {
                LoadData();
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void btnNewGame_Click(object sender, RoutedEventArgs e)
        {
            //Comprobem el nom
            if (txbName.Text == "")
            {
                MessageBox.Show(this, "El nom de la partida no pot estar en blanc", "Error");
                txbName.Focus();
                return;
            }
            if (txbName.Text.Length > 100)
            {
                MessageBox.Show(this, "El nom de la partida no pot superar els 100 caracters", "Error");
                txbName.Focus();
                return;
            }

            //Comprobem que hi hagi algun tauler, sac i diccionari seleccionats
            if (cbbBoard.SelectedIndex == -1)
            {
                MessageBox.Show(this, "Hi ha d'haver un tauler seleccionat", "Error");
                cbbBoard.Focus();
                return;
            }

            if (cbbTileSet.SelectedIndex == -1)
            {
                MessageBox.Show(this, "Hi ha d'haver un sac de fitxes seleccionat", "Error");
                cbbTileSet.Focus();
                return;
            }

            if (cbbDictionary.SelectedIndex == -1)
            {
                MessageBox.Show(this, "Hi ha d'haver un diccionari seleccionat", "Error");
                cbbDictionary.Focus();
                return;
            }

            GameInfoDTO gameInfo = new GameInfoDTO();
            gameInfo.Name = txbName.Text;
            gameInfo.Board = cbbBoard.SelectedValue.ToString();
            gameInfo.TileSet = cbbTileSet.SelectedValue.ToString();
            gameInfo.Dictionary = cbbDictionary.SelectedValue.ToString();

            try
            {
                if (rbtOnePlayer.IsChecked == true)
                {
                    gameInfo = ClientManager.Instance.NewOnePlayerGame(gameInfo);

                    //Si es per un sol jugador ja podem entrar directament a jugar
                    PlayGame pg = new PlayGame();
                    pg.Owner = this.Owner;
                    pg.SetGame(gameInfo, true);
                    this.Visibility = System.Windows.Visibility.Hidden;
                    pg.ShowDialog();
                }
                else
                {
                    gameInfo = ClientManager.Instance.NewTwoPlayersGame(gameInfo);

                    //Si es per dos jugadors portem a la finestra de jocs actuals
                    CurrentGames cg = new CurrentGames();
                    cg.Owner = this.Owner;
                    this.Visibility = System.Windows.Visibility.Hidden;
                    cg.ShowDialog();
                }
             
                this.Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }
    }
}
