﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using ScrabbleClient.Domain;
using ServerLibrary.WCF.DTO;

namespace ScrabbleClient.Interface
{
    /// <summary>
    /// Lógica de interacción para SelectLetter.xaml
    /// </summary>
    public partial class SelectLetter : Window
    {
        private GameInfoDTO gameInfo;
        private string selectedLetter = "";
        private bool canClose = false;

        public SelectLetter()
        {
            InitializeComponent();         
        }

        public GameInfoDTO GameInfo { set { gameInfo = value; } }
        public string SelectedLetter { get { return selectedLetter; } }

        private void frmSelectLetter_Loaded(object sender, RoutedEventArgs e)
        {
            try
            {
                LoadValidLettersGrid();
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void LoadValidLettersGrid()
        {
            List<string> validLetters = ClientManager.Instance.GetValidLetters(gameInfo.Id);

            if (validLetters != null)
            {
                for (int i = 0; i < validLetters.Count(); i++)
                {
                    //Accedim a les caselles del grid per fila i columna mitjançant una consulta LINQ i l'associem a l'objecte intern
                    Button btnTile = grdTiles.Children.Cast<Button>().First(e => Grid.GetRow(e) == (i / 10) && Grid.GetColumn(e) == (i % 10));

                    btnTile.Content = validLetters[i];
                    btnTile.Visibility = System.Windows.Visibility.Visible;
                    btnTile.Click += btnTile_Click;
                }
            }
        }

        private void btnTile_Click(object sender, RoutedEventArgs e)
        {
            selectedLetter = ((Button)e.Source).Content.ToString();
            canClose = true;
            Close();
        }

        private void frmSelectLetter_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            if (!canClose)
            {
                e.Cancel = true;
                MessageBox.Show(this, "Sel·lecciona una fitxa per poder tancar la finestra.", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }     
    }
}
