﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using ServerLibrary.WCF.DTO;
using ScrabbleClient.Domain;

namespace ScrabbleClient.Interface
{
    /// <summary>
    /// Lógica de interacción para ShowStats.xaml
    /// </summary>
    public partial class ShowStats : Window
    {
        private List<StatsHighGameDTO> highestGames = new List<StatsHighGameDTO>();
        private List<StatsHighMoveDTO> highestMoves = new List<StatsHighMoveDTO>();
        private List<StatsGamePercentages> highestPercentages = new List<StatsGamePercentages>();

        public ShowStats()
        {
            InitializeComponent();
        }      

        private void RefreshGrid()
        {
            try
            {
                switch (cbbRankType.SelectedIndex)
                {
                    case 0:
                        RefreshHighestGames(false, false);
                        break;

                    case 1:
                        RefreshHighestGames(false, true);
                        break;

                    case 2:
                        RefreshHighestGames(true, false);
                        break;

                    case 3:
                        RefreshHighestMoves(false, false);
                        break;

                    case 4:
                        RefreshHighestMoves(false, true);
                        break;

                    case 5:
                        RefreshHighestMoves(true, false);
                        break;

                    case 6:
                        RefreshHighestPercentages(false, false);
                        break;

                    case 7:
                        RefreshHighestPercentages(false, true);
                        break;

                    case 8:
                        RefreshHighestPercentages(true, false);
                        break;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        //Mostra la classificació de millors partides
        private void RefreshHighestGames(bool onlyUserStats, bool excludeCPU)
        {
            dgRank.Columns.Clear();

            highestGames = ClientManager.Instance.GetHighestGameScores(onlyUserStats, excludeCPU);

            dgRank.ItemsSource = highestGames;

            DataGridTextColumn playerName = new DataGridTextColumn();
            playerName.Header = "Jugador";
            playerName.Width = new DataGridLength(4, DataGridLengthUnitType.Star);
            playerName.Binding = new Binding("PlayerName");
            dgRank.Columns.Add(playerName);

            DataGridTextColumn gameScore = new DataGridTextColumn();
            gameScore.Header = "Puntuació";
            gameScore.Width = new DataGridLength(2, DataGridLengthUnitType.Star);
            gameScore.Binding = new Binding("GameScore");
            dgRank.Columns.Add(gameScore);

            DataGridTextColumn gameDateTime = new DataGridTextColumn();
            gameDateTime.Header = "Data";
            gameDateTime.Width = new DataGridLength(4, DataGridLengthUnitType.Star);
            gameDateTime.Binding = new Binding("GameDateTime");
            dgRank.Columns.Add(gameDateTime);
        }

        //Mostra la classificació de millors jugades
        private void RefreshHighestMoves(bool onlyUserStats, bool excludeCPU)
        {
            dgRank.Columns.Clear();

            highestMoves = ClientManager.Instance.GetHighestMoveScores(onlyUserStats, excludeCPU);

            dgRank.ItemsSource = highestMoves;

            DataGridTextColumn playerName = new DataGridTextColumn();
            playerName.Header = "Jugador";
            playerName.Width = new DataGridLength(2.5, DataGridLengthUnitType.Star);
            playerName.Binding = new Binding("PlayerName");
            dgRank.Columns.Add(playerName);

            DataGridTextColumn moveScore = new DataGridTextColumn();
            moveScore.Header = "Puntuació";
            moveScore.Width = new DataGridLength(1.5, DataGridLengthUnitType.Star);
            moveScore.Binding = new Binding("MoveScore");
            dgRank.Columns.Add(moveScore);

            DataGridTextColumn moveWords = new DataGridTextColumn();
            moveWords.Header = "Paraules";
            moveWords.Width = new DataGridLength(3.5, DataGridLengthUnitType.Star);
            moveWords.Binding = new Binding("MoveWords");
            dgRank.Columns.Add(moveWords);

            DataGridTextColumn moveDateTime = new DataGridTextColumn();
            moveDateTime.Header = "Data";
            moveDateTime.Width = new DataGridLength(2.5, DataGridLengthUnitType.Star);
            moveDateTime.Binding = new Binding("MoveDateTime");
            dgRank.Columns.Add(moveDateTime);
        }

        //Mostra la classificació de millors percentatges
        private void RefreshHighestPercentages(bool onlyUserStats, bool excludeCPU)
        {
            dgRank.Columns.Clear();

            highestPercentages = ClientManager.Instance.GetHighestWinGamesPercent(onlyUserStats, excludeCPU);

            dgRank.ItemsSource = highestPercentages;

            DataGridTextColumn playerName = new DataGridTextColumn();
            playerName.Header = "Jugador";
            playerName.Width = new DataGridLength(3.4, DataGridLengthUnitType.Star);
            playerName.Binding = new Binding("PlayerName");
            dgRank.Columns.Add(playerName);

            DataGridTextColumn totalWin = new DataGridTextColumn();
            totalWin.Header = "Victories";
            totalWin.Width = new DataGridLength(1, DataGridLengthUnitType.Star);
            totalWin.Binding = new Binding("TotalWin");
            dgRank.Columns.Add(totalWin);

            DataGridTextColumn percentWin = new DataGridTextColumn();
            percentWin.Header = "% Victories";
            percentWin.Width = new DataGridLength(1.2, DataGridLengthUnitType.Star);
            percentWin.Binding = new Binding("PercentWin");
            dgRank.Columns.Add(percentWin);

            DataGridTextColumn totalDraw = new DataGridTextColumn();
            totalDraw.Header = "Empats";
            totalDraw.Width = new DataGridLength(1, DataGridLengthUnitType.Star);
            totalDraw.Binding = new Binding("TotalDraw");
            dgRank.Columns.Add(totalDraw);

            DataGridTextColumn percentDraw = new DataGridTextColumn();
            percentDraw.Header = "% Empats";
            percentDraw.Width = new DataGridLength(1.2, DataGridLengthUnitType.Star);
            percentDraw.Binding = new Binding("PercentDraw");
            dgRank.Columns.Add(percentDraw);

            DataGridTextColumn totalLost = new DataGridTextColumn();
            totalLost.Header = "Derrotes";
            totalLost.Width = new DataGridLength(1, DataGridLengthUnitType.Star);
            totalLost.Binding = new Binding("TotalLost");
            dgRank.Columns.Add(totalLost);

            DataGridTextColumn percentLost = new DataGridTextColumn();
            percentLost.Header = "% Derrotes";
            percentLost.Width = new DataGridLength(1.2, DataGridLengthUnitType.Star);
            percentLost.Binding = new Binding("PercentLost");
            dgRank.Columns.Add(percentLost);
        }

        private void cbbRankType_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            RefreshGrid();
        }

        private void frmShowStats_Loaded(object sender, RoutedEventArgs e)
        {
            RefreshGrid();
        }

        private void btnBack_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }
    }
}
