﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using ServerLibrary.WCF.DTO;
using ScrabbleClient.Domain;

namespace ScrabbleClient.Interface
{
    /// <summary>
    /// Lógica de interacción para SwapTiles.xaml
    /// </summary>
    public partial class SwapTiles : Window
    {
        private string gameId;
        private bool tilesSwaped = false;
        private List<TileDTO> rack;

        public SwapTiles()
        {
            InitializeComponent();
        }

        public string GameId { set { gameId = value; } }
        public bool TilesSwaped { get { return tilesSwaped; } }
        public List<TileDTO> Rack { get { return rack; } set { rack = value; } }

        private void frmSwapTiles_Loaded(object sender, RoutedEventArgs e)
        {
            try
            {
                LoadRack();
            }
            catch (Exception ex)
            {
                MessageBox.Show(this, ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void LoadRack()
        {
            if (rack == null)
            {
                return;
            }

            for (int i = 0; i < rack.Count(); i++)
            {
                //Accedim a les caselles del grid per columna mitjançant una consulta LINQ i l'associem a l'objecte intern
                //I l'imatge que correspongui
                Label tile = grdCurrentRack.Children.Cast<Label>().First(img => Grid.GetColumn(img) == i);

                TileDTO tileDTO = rack[i];

                if (tileDTO != null)
                {
                    tile.Content = tileDTO.Letter;
                    ((ImageBrush)tile.Background).ImageSource = GetTileImageSource(tileDTO.Value);
                }
                else
                {
                    ((ImageBrush)tile.Background).ImageSource = null;
                }

                tile.Tag = rack[i];
            }
        }

        private ImageSource GetTileImageSource(int value)
        {
            switch (value)
            {
                case 0:
                    return img0.Source;

                case 1:
                    return img1.Source;

                case 2:
                    return img2.Source;

                case 3:
                    return img3.Source;

                case 4:
                    return img4.Source;

                case 8:
                    return img8.Source;

                case 10:
                    return img10.Source;

                default:
                    return null;
            }
        }

        private void btnCancel_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        private void btnSave_Click(object sender, RoutedEventArgs e)
        {
            //Recopilem les fitxes seleccionades
            List<TileDTO> selectedTiles = new List<TileDTO>();

            for (int i = 0; i < 7; i++)
            {
                Label tile = grdSelectedRack.Children.Cast<Label>().First(lbl => Grid.GetColumn(lbl) == i);

                if (tile.Tag != null)
                {
                    selectedTiles.Add((TileDTO)tile.Tag);
                }
            }

            //Si no hem seleccionat cap fitxa fem com un cancel
            if (selectedTiles.Count() == 0)
            {
                Close();
            }
            else
            {

                //Demanem confirmació
                if (MessageBox.Show(this, "Estas segur que vols canviar les fitxes seleccionades?", "Confirmació", MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes)
                {
                    try
                    {
                        rack = ClientManager.Instance.SwapTiles(gameId, selectedTiles);
                        tilesSwaped = true;
                        Close();
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(this, ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                    }
                }
            }
        }

        /*Gestió de Drag & drop*/
        private class DragDropData
        {
            public TileDTO Tile { get; set; }
            public Label SourceLabel { get; set; }
        }

        //Métodes del faristol
        private void lblTile_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            //Obtenim l'origen del drag
            Label lbl = (Label)e.Source;

            //Primer comprovem si podem fer drag en aquesta casella del faristol
            if (lbl.Tag != null)
            {
                //Creem l'objecte amb la fitxa que s'ha de desplaçar
                DragDropData data = new DragDropData() { Tile = (TileDTO)lbl.Tag, SourceLabel = lbl };

                //I iniciem el procés de drag
                DragDrop.DoDragDrop(lbl, data, DragDropEffects.Move);
            }
        }

        private void lblTile_DragEnter(object sender, DragEventArgs e)
        {
            //Per si hem de fer algun check
        }

        private void lblTile_Drop(object sender, DragEventArgs e)
        {
            //Obtenim el data que estem traspassant
            DragDropData data = (DragDropData)e.Data.GetData(typeof(DragDropData));

            //Obtenim els components involucrats i les seves fitxes
            Label source = data.SourceLabel;
            TileDTO sourceTile = data.Tile;

            Label dest = (Label)sender;
            TileDTO destTile = (TileDTO)dest.Tag;

            //Passem la fitxa (si n'hi havia) de la casella de destí a la d'origen
            source.Tag = destTile;
            if (destTile != null)
            {
                ((ImageBrush)source.Background).ImageSource = GetTileImageSource(destTile.Value);
                source.Content = destTile.Letter;
            }
            else
            {
                ((ImageBrush)source.Background).ImageSource = null;
                source.Content = "";
            }


            //Passem la fitxa de l'origen al destí
            dest.Tag = sourceTile;
            if (sourceTile != null)
            {
                ((ImageBrush)dest.Background).ImageSource = GetTileImageSource(sourceTile.Value);
                dest.Content = sourceTile.Letter;
            }
            else
            {
                ((ImageBrush)dest.Background).ImageSource = null;
                dest.Content = "";
            }
        }
    }
}
