﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ServerLibrary.Domain.Dictionaries
{
    /**
     * Classe que implementara un diccionari per consultar la validesa de les paraules en forma de diccionari basic
     * Es la forma mes senzilla d'implementar-lo, pero no la mes òptima ni la que ocupa menys memòria.
     * Creada per poder desenvolupar la resta de l'aplicació, pero substituible per alguna implementació millor
     * (Per exemple un diccionari de hash. En principi s'ha avaluat usar un DAWG, però la complexitat d'implementació era molt gran
     * tinguent en compte que no sabem a priori quins caràcters tindrà cada diccionari)
     * 
     * Aquesta classe serveix per desenvolupar i fer proves, ja que permet carregar un arxiu ràpida i fàcilment, però
     * no ens es útil en la implementació final, ja que no es capaç de buscar prefixes, i aquesta es una característica necessaria
     * per la implementació de la IA
     * */
    class BasicDictionary: IScrabbleDictionary
    {
        Dictionary<string, bool> dictionary = new Dictionary<string, bool>();

        public string Id { get; set; }

        public string Name { get; set; }

        public string FileName { get; set; }

        public bool ContainsWord(string word)
        {
            return dictionary.ContainsKey(word);
        }

        public bool ContainsPrefix(string prefix)
        {
            return false;
        }

        public void Initialize()
        {
            using (StreamReader file = new StreamReader(FileName))
            {
                string word;
                while ((word = file.ReadLine()) != null)
                {
                    dictionary.Add(word, true);
                }
            }
        }
    }
}
