﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Collections;
using System.IO;

/*
 * Note: I do not own this code! It was originally written by John Mischel and
 * modified slightly by me.
 * 
 * http://www.devsource.com/c/a/Languages/Bloom-Filters-in-C/
 * 
 * http://dotnetperls.com/
 * Sam Allen, 2008
 * 
 * */

/*
 * Nota Jordi: He trobat aquesta implementació de un diccionari amb un filtre Bloom i l'he adaptat per
 * fer-ne us al projecte.
 * Deixo els credits als autors originals en el comentari superior
 * */

namespace ServerLibrary.Domain.Dictionaries
{
    class BloomFilterDictionary : IScrabbleDictionary
    {
        const int bitCount = 1300000;
        private BitArray hashBits = new BitArray(bitCount);

        public string Id { get; set; }

        public string Name { get; set; }

        public string FileName { get; set; }

        static private int HashString(string stringToHash)
        {
            int hash = 5381;
            for (int i = 0; i < stringToHash.Length; i++)
            {
                hash = ((hash << 5) + hash) + (int)stringToHash[i];
            }
            return hash;
        }

        public bool ContainsWord(string wordIn)
        {
            int hash1 = wordIn.GetHashCode();
            int hashKey1 = Math.Abs(hash1 % bitCount);

            if (hashBits[hashKey1] == false)
            {
                return false;
            }

            int hash2 = HashString(wordIn);
            int hashKey2 = Math.Abs((hash1 + (1 * hash2)) % bitCount);

            if (hashBits[hashKey2] == false)
            {
                return false;
            }

            return true;
        }

        public bool ContainsPrefix(string prefix)
        {
            return false;
        }

        private void Add(string stringIn)
        {
            // set bit 1 from hashkey 1
            int hash1 = stringIn.GetHashCode();
            int hashKey1 = Math.Abs(hash1 % bitCount);
            hashBits[hashKey1] = true;

            // set bit 2 from hashkey 2
            int hash2 = HashString(stringIn);
            int hashKey2 = Math.Abs((hash1 + (1 * hash2)) % bitCount);
            hashBits[hashKey2] = true;
        }

        public void Initialize()
        {
            using (StreamReader file = new StreamReader(FileName))
            {
                string word;
                while ((word = file.ReadLine()) != null)
                {
                    this.Add(word);
                }
            }
        }
    }
}
