﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.Threading.Tasks;

namespace ServerLibrary.Domain.Errors
{
    //Classe que conté els possibles codis d'error que poden llançar les excepcions del programa
    public sealed class ScrabbleError
    {
        /* 
         * Errors relacionats amb els usuaris
         * 0000x
         */
        public const string USER_OR_PASSWORD_BLANK = "00001";
        public const string USER_OR_PASSWORD_INVALID = "00002";
        public const string USER_NAME_ALREADY_EXISTS = "00003";
        public const string USER_NOT_FOUND = "00004";
        public const string USER_NAME_TOO_LONG = "00005";
        public const string USER_HAS_NOT_REQUIRED_PRIVILEGE = "00006";


        /* 
         * Errors relacionats amb els taulers
         * 0010x
         */
        public const string BOARD_NAME_ALREADY_EXISTS = "00103";
        public const string BOARD_NOT_FOUND = "00104";
        public const string BOARD_NAME_TOO_LONG = "00105";
        public const string BOARD_HAS_INVALID_BOX = "00106";
        public const string BOARD_HAS_INVALID_COLUMNS_LENGTH = "00107";
        public const string BOARD_HAS_INVALID_ROWS_LENGTH = "00108";
        public const string BOARD_HAS_INVALID_START_BOXES_NUMBER = "00109";
        public const string BOARD_IS_IN_USE = "00110";
        public const string BOARD_INVALID_BOX_POSITION = "00111";

        /* 
         * Errors relacionats amb els sacs de fitxes
         * 0020x
         */
        public const string TILESET_NAME_ALREADY_EXISTS = "00203";
        public const string TILESET_NOT_FOUND = "00204";
        public const string TILESET_NAME_TOO_LONG = "00205";
        public const string TILESET_HAS_INVALID_TILES_NUMBER = "00206";
        public const string TILESET_IS_IN_USE = "00210";

        /* 
         * Errors relacionats amb els diccionaris
         * 0030x
         */
        public const string DICTIONARY_NAME_ALREADY_EXISTS = "00303";
        public const string DICTIONARY_NOT_FOUND = "00304";
        public const string DICTIONARY_NAME_TOO_LONG = "00305";
        public const string DICTIONARY_FILENAME_TOO_LONG = "00306";
        public const string DICTIONARY_HAS_INVALID_TYPE = "00307";
        public const string DICTIONARY_IS_IN_USE = "00310";

        /* 
         * Errors relacionats amb les partides
         * 0040x
         */
        public const string GAME_NOT_FOUND = "00404"; 
        public const string GAME_PLAYER_TYPE_DOES_NOT_ALLOW_USER = "00405";
        public const string GAME_PLAYER2_ALREADY_EXISTS = "00406";
        public const string GAME_PLAYER2_CAN_NOT_BE_PLAYER1 = "00407";
        public const string GAME_USER_NOT_INVOLVED = "00408";
        public const string GAME_CAN_NOT_START_WITHOUT_PLAYER2 = "00409";
        public const string GAME_TURN_INVALID = "00410";
        public const string GAME_MOVE_NOT_VALID = "00411";
        public const string GAME_INVALID_RACK_POSITION = "00412";


        /* 
         * Errors relacionats amb l'accés a base de dades
         * 1000x
         */
        public const string DATABASE_UNEXPECTED_ERROR = "10000";

        /* 
         * Errors relacionats amb l'accés al sistema d'arxius
         * 2000x
         */
        public const string FILESYSTEM_UNEXPECTED_ERROR = "20000";


        public static FaultException GetNewFaultException(string errorText, string errorCode)
        {
            return new FaultException(errorText, new FaultCode(errorCode));
        }
    }
}
