﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ServerLibrary.Domain.Boards;
using ServerLibrary.Domain.TileSets;
using ServerLibrary.Domain.Errors;
using ServerLibrary.WCF.DTO;

namespace ServerLibrary.Domain.Games
{
    public class Move
    {
        private List<MoveTile> tiles = new List<MoveTile>();
        private List<string> words = new List<string>();
        public enum MoveDirection { Right = 0, Down };

        //Jugador que fa la jugada
        public IPlayer Player { get; set; }

        //Posició inicial de la jugada
        public int InitialRow { get; set; }
        public int InitialColumn { get; set; }

        //Direcció de la paraula principal (nomes pot ser Dreta o Avall)
        public MoveDirection Direction { get; set; }
        
        //Fitxes involucrades en la jugada
        public List<MoveTile> Tiles { get { return tiles; } set { tiles = value; } }

        //Paraules formades en aquesta jugada
        public List<string> Words { get { return words; } set { Words = value; } }

        //Punts totals
        public int Score { get; set; }

        //Carrega les dades de la jugada a partir del objecte DTO, del tauler i del faristol del jugador
        public void FromDTO(MoveDTO moveDTO, Rack rack, Board board)
        {
            //Setejarem les opcions que ens vinguin del client i associarem les fitxes recent jugades
            InitialRow = moveDTO.InitialRow;
            InitialColumn = moveDTO.InitialColumn;
            Direction = (MoveDirection)moveDTO.Direction;

            foreach (MoveTileDTO moveTileDTO in moveDTO.Tiles)
            {
                MoveTile moveTile = new MoveTile() { Row = moveTileDTO.Row, Column = moveTileDTO.Column, Letter = moveTileDTO.Tile.Letter, NewTile = moveTileDTO.NewTile };

                //Si es una nova fitxa la busquem al faristol
                if (moveTile.NewTile)
                {
                    Tile tile = rack.GetTileWithoutRemove(moveTileDTO.Tile.Id);

                    if (tile == null)
                    {
                        ScrabbleError.GetNewFaultException("Jugada invàlida. La jugada no coincideix amb el faristol", ScrabbleError.GAME_MOVE_NOT_VALID);
                    }

                    if (tile.Type == Tile.TileType.Blank)
                    {
                        //Assignem la lletra perque pot ser un escarras que no la tingues assignada
                        tile.Letter = moveTileDTO.Tile.Letter;
                    }
                    moveTile.Tile = tile;
                }
                //Sino la busquem al tauler
                else
                {
                    board.CheckValidBoxPosition(moveTile.Row, moveTile.Column);

                    moveTile.Tile = board.Boxes[moveTile.Row, moveTile.Column].Tile;

                    if (moveTile.Tile == null || moveTile.Tile.Id != moveTileDTO.Tile.Id)
                    {
                        ScrabbleError.GetNewFaultException("Jugada invàlida. La jugada no coincideix amb el tauler", ScrabbleError.GAME_MOVE_NOT_VALID);
                    }
                }

                Tiles.Add(moveTile);
            }
        }
    
        //Copia les dades de l'objecte a un objecte nou
        public Move Clone()
        {
            Move result = new Move();
            result.Player = Player; //No el clonem, referència al jugador

            result.InitialColumn = InitialColumn;
            result.InitialRow = InitialRow;
            result.Direction = Direction;
            result.Score = Score;

            result.words.AddRange(words);

            foreach (MoveTile tile in tiles)
            {
                result.Tiles.Add(tile.Clone());
            }

            return result;
        }

        //Per debugar
        public override string ToString()
        {
            string result = "";

            result += "   Direcció: " + (Direction == MoveDirection.Right? "Horitzontal": "Vertical") + "\n";
            result += "       Fila: " + InitialRow + "\n";
            result += "    Columna: " + InitialColumn + "\n\n";
            result += "      Punts: " + Score + "\n\n";

            string word = "";
            foreach(MoveTile mt in tiles)
            {
                word += mt.Tile.Letter;
            }

            result += "      Fitxes: " + word + "\n";
            result += "    Paraules: " + string.Join<string>(", ", words) + "\n\n\n";

            return result;
        }
    }
}
