﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ServerLibrary.Domain.TileSets;
using ServerLibrary.Domain.Errors;
using ServerLibrary.WCF.DTO;

namespace ServerLibrary.Domain.Games
{
    //Classe que representa un faristol d'un jugador
    public class Rack
    {
        public const int RACK_TILES = 7;

        private Tile[] tiles = new Tile[RACK_TILES];

        public int Count()
        {
            int result = 0;
            for (int i = 0; i < RACK_TILES; i++)
            {
                if (tiles[i] != null)
                {
                    result++;
                }
            }

            return result;
        }

        //Retorna la suma total dels valors de les fitxes del faristol
        public int GetTotalPoints()
        {
            int result = 0;
            for (int i = 0; i < RACK_TILES; i++)
            {
                if (tiles[i] != null)
                {
                    result += tiles[i].Value;
                }
            }

            return result;
        }

        //Agafara les fitxes que necessiti del sac de fitxes
        public void TakeTiles(TileSet tileSet)
        {
            for (int i = 0; i < tiles.Length; i++)
            {
                if (tiles[i] == null)
                {
                    tiles[i] = tileSet.GetNextTile();
                }
            }
        }

        //Torna totes les fitxes que coincideixen amb les passades per paràmetre
        public List<Tile> GetTiles(List<TileDTO> listDTO)
        {
            List<Tile> result = new List<Tile>();

            foreach (TileDTO tileDTO in listDTO)
            {
                Tile tile = GetTile(tileDTO.Id);
                if (tile != null)
                {
                    result.Add(tile);
                }
            }

            return result;
        }

        public Tile GetTile(string tileId)
        {
            Tile result;
            for (int i = 0; i < tiles.Length; i++)
            {
                if (tiles[i] != null && tiles[i].Id == tileId)
                {
                    result = tiles[i];
                    tiles[i] = null;
                    return result;
                }
            }
            return null;
        }

        public Tile GetTileWithoutRemove(string tileId)
        {
            for (int i = 0; i < tiles.Length; i++)
            {
                if (tiles[i] != null && tiles[i].Id == tileId)
                {
                    return tiles[i];
                }
            }
            return null;
        }

        public Tile GetTile(int tilePos)
        {
            Tile result = tiles[tilePos];
            tiles[tilePos] = null;
            return result;
        }

        public Tile GetTileWithoutRemove(int tilePos)
        {
            if (tilePos < 0 || tilePos >= tiles.Length)
            {
                ScrabbleError.GetNewFaultException(String.Format("Posició invalida al faristol: {0}", tilePos), ScrabbleError.GAME_INVALID_RACK_POSITION);
            }

            return tiles[tilePos];
        }

        //Mètode que torna una fitxa si conté la lletra especificada o es un escarràs (en cas que hi hagi dels dos tipus es dona preferencia a la fitxa normal)
        public Tile GetIfContains(string letter)
        {
            Tile result = null;

            for (int i = 0; i < tiles.Length; i++)
            {          
                if (tiles[i] != null)
                {
                    //Si la fitxa te la lletra que busquem la tornem
                    if (tiles[i].Letter == letter)
                    {
                        result = tiles[i];
                        tiles[i] = null;
                        return result;
                    }

                    //Si es el primer escarràs que trobem el desem a result
                    if (tiles[i].Type == Tile.TileType.Blank && result == null)
                    {
                        result = tiles[i];
                        tiles[i] = null;
                    }
                }
            }

            //Si hem arribat aqui es que no hem trobat la lletra explicitament i
            //o be tenim un escarràs o be res, en tot cas tornem result
            return result;
        }

        //Retorna cert si hi ha alguna fitxa que sigui escarràs o la seva lletra estigui dins del conjunt passat per paràmetre
        public bool HasAnyValidTile(Dictionary<string, bool> validTiles)
        {
            //Reto
            for (int i = 0; i < RACK_TILES; i++)
            {
                if(tiles[i] != null && (tiles[i].Type == Tile.TileType.Blank || validTiles.ContainsKey(tiles[i].Letter)))
                {
                    return true;
                }
            }
            return false;
        }

        public void SetTile(int tilePos, Tile tile)
        {
            if (tilePos < 0 || tilePos >= tiles.Length)
            {
                ScrabbleError.GetNewFaultException(String.Format("Posició invalida al faristol: {0}", tilePos), ScrabbleError.GAME_INVALID_RACK_POSITION);
            }

            tiles[tilePos] = tile;
        }

        public void ProcessMove(Move move)
        {
            //Per cada fitxa de la jugada
            foreach (MoveTile moveTile in move.Tiles)
            {
                //Si la fitxa es nova, la treiem del tauler
                if (moveTile.NewTile)
                {
                    GetTile(moveTile.Tile.Id);
                }
            }
        }   

        public List<TileDTO> ToDTO()
        {
            List<TileDTO> result = new List<TileDTO>();

            for (int i = 0; i < tiles.Length; i++)
            {
                TileDTO tileDTO = null;
                if (tiles[i] != null)
                {
                    tileDTO = tiles[i].ToDTO();
                }
                result.Add(tileDTO);
            }

            return result;
        }
    }
}
