﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ServerLibrary.Domain.TileSets
{
    public class TileSet
    {
        public const int NUMBER_OF_TILES = 100;

        //Llista de fitxes
        private List<Tile> tiles = new List<Tile>();
        private SortedDictionary<string, string> validLetters = new SortedDictionary<string, string>();

        public string Id { get; set; }
        public string Name { get; set; }
        public DateTime Date { get; set; }
        public List<Tile> Tiles { get { return tiles; } }
        public List<string> ValidLetters { get { return validLetters.Keys.ToList(); } }


        public void Shuffle()
        {
            tiles = tiles.OrderBy(a => Guid.NewGuid()).ToList<Tile>(); ;
        }

        //Retorna la següent fitxa del sac
        public Tile GetNextTile()
        {
            if (tiles.Count > 0)
            {
                Tile tile = tiles[tiles.Count - 1];
                tiles.RemoveAt(tiles.Count - 1);
                return tile;
            }
            else
            {
                return null;
            }
        }

        //Afegeix fitxes al sac
        public void AddTiles(List<Tile> tiles)
        {
            this.tiles.AddRange(tiles);
        }

        public void AddTile(Tile tile)
        {
            tiles.Add(tile);
            //Desem les diferents lletres que van apareixent. Aquestes seran les úniques lletres vàlides
            //quan un jugador vulgui usar un escarràs
            if (tile.Letter != "")
            {
                validLetters[tile.Letter] = tile.Letter;
            }
        }

        //Treu la fitxa amb l'id corresponent del sac
        public Tile GetTile(string tileId)
        {
            if (tileId == null)
            {
                return null;
            }

            int i = 0;
            while (i < tiles.Count())
            {
                if (tiles[i].Id == tileId)
                {
                    Tile result = tiles[i];
                    tiles.RemoveAt(i);
                    return result;
                }
                i++;
            }
            return null;
        }
    }
}
