﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ServerLibrary.Utils
{
    /* Aquesta implementació pren com a base la següent implementació trobada a internet:
     * http://counterhelix.com/2011/08/16/basic-trie-data-structure-implementation
     * però esta adaptada i modificada (i una mica optimitzada) per cobrir les necessitats d'aquest projecte
     */
    //Classe que implementa un diccionari en forma d'arbre tipus "Trie"
    public class Trie
    {
        private TrieNode root;

        public Trie()
        {
            root = new TrieNode(' ');
        }

        public TrieNode Root { get { return root; } }

        public void Insert(string s)
        {
            char[] word = s.ToCharArray();

            TrieNode currentNode = root;

            if (word.Length == 0)
            {
                currentNode.IsValidEnding = true;
            }

            //Per cada lletra de la paraula...
            for (int i = 0; i < s.Length; i++)
            {
                //... busquem en els fills del node actual la seguent lletra
                TrieNode childNode = currentNode.ChildNode(word[i]);

                //Si el trobem desem el node actual
                if (childNode != null)
                {
                    currentNode = childNode;
                }
                //En cas contrari creem el nou node fill i el desem com a actual.
                else
                {
                    currentNode = currentNode.Add(word[i]);
                }

                //Si es l'ultima lletra marquem el node com a terminal valid
                if (i == word.Length - 1)
                {
                    currentNode.IsValidEnding = true;
                }
            }
        }

        //Mètode que buscarà una paraula completa
        public bool SearchWord(string s)
        {
            return Search(s, false);
        }

        //Mètode que buscarà un prefix
        public bool SearchPrefix(string s)
        {
            return Search(s, true);
        }


        private bool Search(string s, bool validPrefix)
        {
            char[] word = s.ToCharArray();

            TrieNode currentNode = root;

            if (currentNode != null)
            {
                //Per cada lletra de la paraula
                for (int i = 0; i < word.Length; i++)
                {
                    //Busquem el fill que conté la lletra actual
                    currentNode = currentNode.ChildNode(word[i]);
                    //Si no hi es, la paraula no es vàlida
                    if (currentNode == null)
                    {
                        return false;
                    }
                }

                //Arribats aqui hem trobat tota la seqüencia de lletres
                //Tornarem true si es un final valid o si buscavem prefixes
                if (currentNode.IsValidEnding || validPrefix)
                {
                    return true;
                }
                else
                {
                    return false;
                }
            }

            return false;
        }

    }
}
