﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;
using ServerLibrary.WCF.DTO;


namespace ServerLibrary
{
    [ServiceContract(Namespace = "http://www.JordiSimon/.net/scrabble/2013/04",
                     Name = "IScrabbleService",
                     SessionMode = SessionMode.Required,
                     CallbackContract = typeof(IScrabbleServiceCallback))]
    public interface IScrabbleService
    {
        //WCF no suporta matrius multidimensionals ([,]), nomes unidimensionals ([]) o escalonades ([][])
        //pero a més hem canviat el tipus per defecte de les coleccions amb que treballen els metodes de WCF
        //de Array a List<T>, per tant una matriu T[,] passa a ser List<List<T>>

        //Manté la sessió oberta
        [OperationContract(IsOneWay = true, IsInitiating = false, IsTerminating = false)]
        void HeartBeat();


        /*
         * *************************************************************************************
         * GESTIÓ D'USUARIS
         * *************************************************************************************
         */

        //Inicia sessió i es desa una referencia al callback
        [OperationContract(IsInitiating = true, IsTerminating = false)]
        UserDTO LogIn();

        //Finalitza la sessió
        [OperationContract(IsInitiating = false, IsTerminating = true)]
        void LogOut();

        //Crea un nou usuari
        [OperationContract(IsInitiating = true, IsTerminating = true)]
        void CreateUser(UserDTO userDTO);

        //Actualitza les dades de l'usuari
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        void UpdateUser(UserDTO userDTO);

        //Elimina l'usuari (i les seves estadístiques si s'escau)
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        void DeleteUser(bool deleteStats);


        /*
         * *************************************************************************************
         * GESTIÓ DE TAULERS
         * *************************************************************************************
         */

        //Retorna la llista de taulers de l'usuari identificat
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        List<BoardDTO> GetUserBoards();

        //Retorna la llista de taulers disponibles (els del usuari mes els que hi hagi per defecte)
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        List<BoardDTO> GetBoards();

        //Retorna la matriu de caselles d'un tauler determinat
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        List<List<BoxDTO>> GetBoardBoxes(string boardId);

        //Desa un tauler a la base de dades
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        void SaveBoard(BoardDTO board, List<List<BoxDTO>> boxes);

        //Elimina un tauler de la base de dades
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        void DeleteBoard(BoardDTO board);


        /*
         * *************************************************************************************
         * GESTIÓ DE SACS DE FITXES
         * *************************************************************************************
         */

        //Retorna la llista de sacs de fitxes de l'usuari identificat
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        List<TileSetDTO> GetUserTileSets();

        //Retorna la llista de sacs de fitxesdisponibles (els del usuari mes els que hi hagi per defecte)
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        List<TileSetDTO> GetTileSets();

        //Retorna la llista de fitxes d'un sac de fitxes determinat
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        List<TileDTO> GetTileSetTiles(string tileSetId);

        //Desa un sac de fitxes a la base de dades
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        void SaveTileSet(TileSetDTO tileSet, List<TileDTO> tiles);

        //Elimina un sac de fitxes dela base de dades
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        void DeleteTileSet(TileSetDTO tileSet);

        /*
        * *************************************************************************************
        * GESTIÓ DE DICCIONARIS
        * *************************************************************************************
        */

        //Retorna la llista de informació de diccionaris (gestió nomes per usuaris administradors)
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        List<DictionaryInfoDTO> GetDictionaryInfos();

        //Desa una informació de diccionari a la base de dades (gestió nomes per usuaris administradors)
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        void SaveDictionaryInfo(DictionaryInfoDTO dictionaryInfo);

        //Elimina una informació de diccionari de la base de dades (gestió nomes per usuaris administradors)
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        void DeleteDictionaryInfo(DictionaryInfoDTO dictionaryInfo);


        /*
         * *************************************************************************************
         * GESTIÓ DE PARTIDES
         * *************************************************************************************
         */
        //Crea una nova partida per un sol jugador
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        GameInfoDTO NewOnePlayerGame(GameInfoDTO gameInfo);

        //Crea una nova partida per dos jugadors
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        GameInfoDTO NewTwoPlayersGame(GameInfoDTO gameInfo);

        //Elimina una partida
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        void DeleteGame(string gameId);

        //Busca les partides pendents d'afegir un segon jugador
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        List<GameInfoDTO> GetAvailableGames();

        //Busca les partides en que participa l'usuari
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        List<GameInfoDTO> GetCurrentUserGames();

        //Afegeix a l'usuari com a segon jugador d'una partida
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        GameInfoDTO AddMeAsSecondPlayer(string gameId);

        //Retorna la informació bàsica d'una partida actualitzada
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        GameInfoDTO GetGameInfo(string gameId);

        //Retorna l'estat del tauler en un moment determinat
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        List<List<BoxDTO>> GetGameBoardStatus(string gameId);

        //Retorna el faristol d'una partida del jugador
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        List<TileDTO> GetGameRack(string gameId);

        //Senyal indicant que es pot iniciar la partida
        [OperationContract(IsInitiating = false, IsTerminating = false, IsOneWay = true)]
        void StartGame(string gameId);

        //Indica que l'usuari s'està adjuntant a la partida (retorna si es el torn o no de l'usuari)
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        bool AttachToGame(string gameId);

        //Renuncia al torn i el passa l'altre jugador
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        void PassTurn(string gameId);

        //Canvia les fitxes per unes altres del sac
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        List<TileDTO> SwapTiles(string gameId, List<TileDTO> tiles);

        //Retorna la llista de lletres vàlides del sac de fitxes associat a una partida
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        List<string> GetValidLetters(string gameId);

        //L'usuari es rendeix i dona com a guanyador el rival
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        void Resign(string gameId);

        //Comprova els punts que dona una jugada
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        MoveInfoDTO CheckMove(MoveDTO move);

        //Realitza una jugada
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        MoveInfoDTO MakeMove(MoveDTO move);


        /*
         * *************************************************************************************
         * CONSULTA D'ESTADÍSTIQUES
         * *************************************************************************************
         */

        //Torna el Top10 de de millors partides
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        List<StatsHighGameDTO> GetHighestGameScores(bool onlyUserStats, bool excludeCPU);

        //Torna el Top10 de de millors jugades
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        List<StatsHighMoveDTO> GetHighestMoveScores(bool onlyUserStats, bool excludeCPU);

        //Torna el Top10 de de millors percentatges de partides guanyades
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        List<StatsGamePercentages> GetHighestWinGamesPercent(bool onlyUserStats, bool excludeCPU);

        //Elimina les estadístiques de l'usuari
        [OperationContract(IsInitiating = false, IsTerminating = false)]
        void DeleteUserStats();
    }


    public interface IScrabbleServiceCallback
    {
        [OperationContract(IsOneWay = true)]
        void YourTurn(GameInfoDTO gameInfo, List<List<BoxDTO>> boardInfo, List<TileDTO> rackInfo, MoveInfoDTO moveInfo);

        [OperationContract(IsOneWay = true)]
        void OpponentResigned(GameInfoDTO gameInfo);
    }
}
